"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const graphql_config_1 = require("graphql-config");
const config_1 = require("../src/config");
jest.mock('fs', () => {
    return {
        existsSync: jest.fn(),
    };
});
jest.mock('glob', () => jest.fn());
const existsSync = jest.requireMock('fs').existsSync;
const glob = jest.requireMock('glob');
const configData = { schemaPath: 'test' };
const configPath = path_1.join(__dirname, '.graphqlconfig');
describe('resolvePathRelativeToConfig()', () => {
    it('resolves a relative path', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        expect(config_1.resolvePathRelativeToConfig(projectConfig, 'test')).toBe(path_1.join(__dirname, 'test'));
    });
    it('resolves an absolute path', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        expect(config_1.resolvePathRelativeToConfig(projectConfig, '/test')).toBe('/test');
    });
});
describe('resolveProjectName()', () => {
    it('resolves a named project name', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath, 'test');
        expect(config_1.resolveProjectName(projectConfig)).toBe('test');
    });
    it('ignores the provided defaultName for a named project', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath, 'test');
        expect(config_1.resolveProjectName(projectConfig, 'ignored')).toBe('test');
    });
    it('uses the provided defaultName for a nameless project', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        expect(config_1.resolveProjectName(projectConfig, 'test')).toBe('test');
    });
    it('uses default projectName for a nameless project when defaultName is omitted', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        expect(config_1.resolveProjectName(projectConfig)).toBe(config_1.defaultGraphQLProjectName);
    });
});
describe('resolveSchemaPath()', () => {
    beforeEach(() => {
        existsSync.mockClear();
    });
    it('throws an error if the schemaPath is empty', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig({}, configPath);
        expect(() => config_1.resolveSchemaPath(projectConfig)).toThrow(/Missing GraphQL schemaPath/i);
    });
    it('throws an error if the schemaPath does not exist', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        existsSync.mockImplementation(() => false);
        expect(() => config_1.resolveSchemaPath(projectConfig)).toThrow(/Schema not found/i);
    });
    it('returns the schemaPath if it exists', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        existsSync.mockImplementation(() => true);
        expect(config_1.resolveSchemaPath(projectConfig)).toBe(path_1.join(__dirname, configData.schemaPath));
        expect(existsSync).toHaveBeenCalledWith(path_1.join(__dirname, configData.schemaPath));
    });
    it('returns the non-existent schemaPath when ignoreMissing is true', () => {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig(configData, configPath);
        existsSync.mockImplementation(() => false);
        expect(config_1.resolveSchemaPath(projectConfig, true)).toBe(path_1.join(__dirname, configData.schemaPath));
    });
});
describe('getGraphQLProjects()', () => {
    it('returns all projects in a multi-project configuration', () => {
        const config = new graphql_config_1.GraphQLConfig({
            schemaPath: '',
            projects: { foo: { schemaPath: 'foo' }, bar: { schemaPath: 'bar' } },
        }, configPath);
        const projects = config_1.getGraphQLProjects(config);
        expect(projects).toHaveLength(2);
        expect(projects[0].projectName).toBe('foo');
        expect(projects[0].schemaPath).toBe(path_1.join(__dirname, 'foo'));
        expect(projects[1].projectName).toBe('bar');
        expect(projects[1].schemaPath).toBe(path_1.join(__dirname, 'bar'));
    });
    it('returns one project in a single project configuration', () => {
        const config = new graphql_config_1.GraphQLConfig({
            schemaPath: 'single',
        }, configPath);
        const projects = config_1.getGraphQLProjects(config);
        expect(projects).toHaveLength(1);
        expect(projects[0].schemaPath).toBe(path_1.join(__dirname, 'single'));
    });
    it('throws an error if no projects are defined', () => {
        const config = new graphql_config_1.GraphQLConfig({ projects: {} }, configPath);
        expect(() => config_1.getGraphQLProjects(config)).toThrow(/No projects defined/i);
    });
});
describe('getGraphQLSchemaPaths()', () => {
    beforeEach(() => {
        existsSync.mockClear();
    });
    it('returns schemaPath for each project', () => {
        const config = new graphql_config_1.GraphQLConfig({
            schemaPath: '',
            projects: { foo: { schemaPath: 'foo' }, bar: { schemaPath: 'bar' } },
        }, configPath);
        existsSync.mockImplementation(() => true);
        expect(config_1.getGraphQLSchemaPaths(config)).toStrictEqual(expect.arrayContaining([path_1.join(__dirname, 'foo'), path_1.join(__dirname, 'bar')]));
    });
});
describe('getGraphQLProjectIncludedFilePaths()', () => {
    beforeEach(() => {
        glob.mockClear();
    });
    it('joins all file paths from each included pattern', () => __awaiter(void 0, void 0, void 0, function* () {
        const projectConfig = new graphql_config_1.GraphQLProjectConfig({
            schemaPath: 'test',
            includes: ['app/A/**/*', 'app/B/**/*'],
            excludes: ['**/excluded'],
        }, configPath);
        // eslint-disable-next-line no-empty-pattern
        glob.mockImplementationOnce(({}, {}, cb) => cb(null, ['fileA']));
        // eslint-disable-next-line no-empty-pattern
        glob.mockImplementationOnce(({}, {}, cb) => cb(null, ['fileB']));
        const filePaths = yield config_1.getGraphQLProjectIncludedFilePaths(projectConfig);
        expect(filePaths).toHaveLength(2);
        expect(filePaths).toStrictEqual(expect.arrayContaining(['fileA', 'fileB']));
        expect(glob).toHaveBeenCalledTimes(2);
        expect(glob.mock.calls).toStrictEqual(expect.arrayContaining([
            [
                path_1.join(__dirname, 'app/A/**/*'),
                {
                    ignore: [path_1.join(__dirname, '**/excluded')],
                },
                expect.any(Function),
            ],
            [
                path_1.join(__dirname, 'app/B/**/*'),
                {
                    ignore: [path_1.join(__dirname, '**/excluded')],
                },
                expect.any(Function),
            ],
        ]));
    }));
});
describe('getGraphQLProjectForSchemaPath()', () => {
    it('returns the schema in a multi-project configuration', () => {
        const config = new graphql_config_1.GraphQLConfig({
            schemaPath: '',
            projects: { foo: { schemaPath: 'foo' } },
        }, configPath);
        const projectConfig = config_1.getGraphQLProjectForSchemaPath(config, path_1.join(__dirname, 'foo'));
        expect(projectConfig.schemaPath).toBe(path_1.join(__dirname, 'foo'));
    });
    it('returns the schema in a single project configuration', () => {
        const config = new graphql_config_1.GraphQLConfig({
            schemaPath: 'foo',
        }, configPath);
        const projectConfig = config_1.getGraphQLProjectForSchemaPath(config, path_1.join(__dirname, 'foo'));
        expect(projectConfig.schemaPath).toBe(path_1.join(__dirname, 'foo'));
    });
    it('throws an error if the schemaPath does not match any project', () => {
        const config = new graphql_config_1.GraphQLConfig({
            schemaPath: 'foo',
        }, configPath);
        expect(() => config_1.getGraphQLProjectForSchemaPath(config, path_1.join(__dirname, 'bar'))).toThrow(/No project defined/i);
    });
});
